@@ -8,8 +8,12 @@ def enhanced_dir(arg, categorize=True, show_types=False, checks=False, collectio
     failed_ = defaultdict(lambda: defaultdict(set))
     x = arg
 
-    for method in dir(arg):
-        type_ = type(eval(f'x.{method}'))
+    for method in (set(dir(arg)) | (set(dir(type(x))) - set(dir(x)))):
+        try:
+            type_ = type(eval(f'x.{method}'))
+        except:
+            failed[f'{arg}'].add(method)
+            continue
         try:
             qualname = eval(f'x.{method}.__qualname__')
             qualname = qualname.split('.')
@@ -72,8 +76,8 @@ def two_way(operation, opposite=False, iterators=False):
     from collections import defaultdict
     failed = defaultdict(set)
     succeeded = defaultdict(set)
-    invalid = 'Error|Warning|Exception|Exit|Interrupt|__|ipython|display|execfile|dreload|help|license|open' \
-              '|get_ipython|credits|runfile|copyright|breakpoint|input|print'
+    invalid = 'StopIteration|StopAsyncIteration|Error|Warning|Exception|Exit|Interrupt|__|ipython|display|execfile' \
+              '|dreload|help|license|open|get_ipython|credits|runfile|copyright|breakpoint|input|print'
     bytes_iterator = "(iter(b''))"
     bytearray_iterator = "(iter(bytearray()))"
     dict_keyiterator = "(iter({}.keys()))"
@@ -88,11 +92,14 @@ def two_way(operation, opposite=False, iterators=False):
     zip_iterator = "(iter(zip()))"
     line_iterator = "(lambda x: 1).__code__.co_lines"
     positions_iterator = "(lambda x: 1).__code__.co_positions"
+    mappingproxy = '(type.__dict__)'
+    generator = '((lambda: (yield))())'
     ## views ##
     dict_keys = 'dict().keys'
     dict_values = 'dict().values'
     dict_items = 'dict().items'
-    y = [(dict_keys, 13), (dict_values, 14), (dict_items, 15)]
+    y = [(dict_keys, 13), (dict_values, 14), (dict_items, 15), (mappingproxy, 16),
+         (generator, 17)]
     if iterators:
         y += [(bytes_iterator, 0), (bytearray_iterator, 1), (dict_keyiterator, 2),
               (dict_valueiterator, 3), (dict_itemiterator, 4), (list_iterator, 5),
@@ -100,11 +107,8 @@ def two_way(operation, opposite=False, iterators=False):
               (set_iterator, 9), (str_iterator, 10), (tuple_iterator, 11), (zip_iterator, 12)]
 
     for a, i in list(keyword.__builtins__.items()) + y:
-        a = str(a)
         if not re.search(invalid, str(a)):
             for b, j in list(keyword.__builtins__.items()) + y:
-                b = str(b)
-                t = t1 = t2 = 0
                 if not re.search(invalid, b):
                     try:
                         x = eval(f'{a}() {operation} {b}()')
@@ -113,7 +117,6 @@ def two_way(operation, opposite=False, iterators=False):
                         else:
                             succeeded[f'{a}()'].add(f'{b}()')
                     except:
-                        t = 1
                         failed[a].add(b)
                     try:
                         x = eval(f'{a}() {operation} {b}')
@@ -122,7 +125,6 @@ def two_way(operation, opposite=False, iterators=False):
                         else:
                             succeeded[f'{a}()'].add(b)
                     except:
-                        t1 = 1
                         failed[a].add(b)
                     try:
                         x = eval(f'{a} {operation} {b}()')
@@ -131,17 +133,15 @@ def two_way(operation, opposite=False, iterators=False):
                         else:
                             succeeded[a].add(f'{b}()')
                     except:
-                        t2 = 1
                         failed[a].add(b)
-                    if t and t1 and t2:
-                        try:
-                            x = eval(f'{a} {operation} {b}')
-                            if opposite:
-                                succeeded[b].add(a)
-                            else:
-                                succeeded[a].add(b)
-                        except:
-                            failed[a].add(b)
+                    try:
+                        x = eval(f'{a} {operation} {b}')
+                        if opposite:
+                            succeeded[b].add(a)
+                        else:
+                            succeeded[a].add(b)
+                    except:
+                        failed[a].add(b)
     return [{'succeeded': succeeded}]
 
 
